@@ -12,13 +12,17 @@ from selenium.webdriver.common.keys import Keys
 from zenpy import Zenpy
 from zenpy.lib.api_objects import CustomField, Ticket, Comment, User
 
-from zenpy.lib.exception import RecordNotFoundException, APIException
+from zenpy.lib.exception import APIException
 from requests.exceptions import RequestException, ConnectionError, Timeout
 from selenium.common.exceptions import WebDriverException, NoSuchElementException, StaleElementReferenceException, TimeoutException
 
 from dataclasses import dataclass
 from typing import Optional, Dict, Any, Iterable, List
 import traceback
+import logging
+from zoneinfo import ZoneInfo
+import datetime
+import json
 
 import requests
 import urllib3
@@ -36,20 +40,29 @@ erro = 'ERRO'
 manutencao = 'EM MANUTENÇÃO'
 desligado = 'DESLIGADO'
 
-__all__ = ['RecordNotFoundException', 
-           'APIException',
-           'RequestException',
-           'ConnectionError',
-           'Timeout',
-           'WebDriverException',
-           'NoSuchElementException',
-           'StaleElementReferenceException',
-           'TimeoutException'
+__all__ = [
+        #Exceções
+           'APIException', 'RequestException', 'ConnectionError', 'Timeout', 'WebDriverException', 
+           'NoSuchElementException', 'StaleElementReferenceException', 'TimeoutException',
+        #Bibliotecas
+           'sleep', 'requests', 'urllib3',
+        #Selenium
+           'webdriver', 'ChromeOptions', 'FirefoxOptions', 'Service', 'By', 'WebDriverWait', 'EC', 'Select',
+           'ActionChains', 'Keys',
+        #Zenpy
+           'Zenpy', 'CustomField', 'Ticket', 'Comment',
+           'User',
+        #Classes
+           'Heartbeat', 'Driver_Selenium', 'Zendesk_Selenium', 'Zendesk_Zenpy', 'BotsLogger',
+        #Variaveis
+           'execucao', 'aguardando', 'erro', 'manutencao', 'desligado'
            ]
 
 class Heartbeat():
     """Destinada a enviar atualizações via API para o painel de bots."""
     def __init__(self, bot_id, endpoint, token):
+        if not all([bot_id, endpoint, token]):
+            raise ValueError("bot_id, endpoint, e token são obrigatórios para o Heartbeat.")
         self.bot_id = bot_id
         self.endpoint = endpoint
         self.token = token
@@ -371,6 +384,8 @@ class Zendesk_Selenium:
             actions.key_down(Keys.CONTROL).key_down(Keys.ALT).send_keys('d').key_up(Keys.ALT).key_up(Keys.CONTROL).perform()
         elif status_ajustado == 'pendente':
             actions.key_down(Keys.CONTROL).key_down(Keys.ALT).send_keys('p').key_up(Keys.ALT).key_up(Keys.CONTROL).perform()
+        elif status_ajustado == 'mesmo' or status_ajustado == 'mesmo status':
+            actions.key_down(Keys.CONTROL).key_down(Keys.ALT).send_keys('u').key_up(Keys.ALT).key_up(Keys.CONTROL).perform()
 
     def fechar_ticket_atual(self):
         """Fecha o ticket atual para evitar cacheamento no driver durante as iterações em cada ticket."""
@@ -562,9 +577,6 @@ class Zendesk_Zenpy:
                 resultado[nome] = valor
 
             return resultado
-        except RecordNotFoundException:
-            print(f"Ticket {ticket_id} não encontrado no Zendesk.")
-            return None
         except APIException as e:
             print(f"Erro de API ao buscar ticket {ticket_id}: {e}")
             return None
@@ -575,3 +587,101 @@ class Zendesk_Zenpy:
 
 #=============================================================================
 
+
+class BotsLogger:
+    def __init__(self, heartbeat_instancia: Heartbeat):
+        if not isinstance(heartbeat_instancia, Heartbeat):
+            raise TypeError("A BotsLogger precisa receber uma instância da classe Heartbeat.")
+        self.heartbeat = heartbeat_instancia
+        self.bot_id = self.heartbeat.bot_id
+
+    def error(self,
+              error: Exception, 
+              message: str, 
+              ticket_id : str,
+              error_type: str = 'WARNING'):
+        """
+        Log de erros, classificando-os e decidindo ações.
+        error_type pode ser: 'WARNING', 'CRITICAL', 'API_RETURN'
+        """
+        error_class = error.__class__.__name__
+        error_message = str(error)
+        stack_trace = traceback.format_exc()
+
+        details = {
+            "ticket_id": ticket_id,
+            "error_type": error_type.upper(),
+            "error_class": error_class,
+            "error_message": error_message,
+            "stack_trace": stack_trace
+        }
+
+        # Lógica de classificação e ação (evolução do seu BotError)
+        level, action = self._classify_error(error, error_type, message)
+        
+        self._log(level, message, details)
+        
+        return action
+        
+
+    def _log(self, level: str, message: str, context: dict):
+        """Método base para criar a estrutura do log."""
+        SAO_PAULO_TZ = ZoneInfo("America/Sao_Paulo")
+        timestamp_sp = datetime.datetime.now(datetime.UTC).astimezone(SAO_PAULO_TZ)
+
+        log_entry = {
+            "level": level.upper(),
+            "message": message,
+            "timestamp": timestamp_sp.isoformat(),
+            "service": self.bot_id,
+            "context": context
+        }
+        # Converte para string JSON e loga
+        logging.log(logging.getLevelName(level.upper()), json.dumps(log_entry, default=str))
+
+        self._envia_alerta(log_entry)
+
+    def _envia_alerta(self, log):
+        """Metodo base para enviar alertas"""
+        level = log.get('level')
+        if level == 'CRITICAL':
+            self.heartbeat.alertas(status=manutencao, warning=log)
+        else:
+            self.heartbeat.alertas(warning=log)
+
+    def _classify_error(self, error: Exception, error_type: str, context: str):
+        """Classifica o erro e retorna (level, action)."""
+        error_message_lower = str(error).lower()
+
+        # Erros Críticos (requerem intervenção imediata)
+        if "invalid credentials" in error_message_lower or "acesso revogado" in error_message_lower:
+            return "CRITICAL", "stop"
+
+        # Erros de API que podem ser tentados novamente
+        if isinstance(error, requests.exceptions.Timeout) or isinstance(error, requests.exceptions.ConnectionError):
+            return "WARNING", "retry"
+        
+        if isinstance(error, requests.exceptions.HTTPError):
+            status_code = error.response.status_code
+            if 500 <= status_code < 600:
+                # Erro no servidor (500, 502, 503...). Pode ser temporário.
+                return "WARNING", "retry" 
+            elif 400 <= status_code < 500:
+                # Erro do cliente (401, 403, 404...). Erro nosso, não adianta tentar de novo.
+                return "CRITICAL", "stop"
+
+        # Erros do Selenium, provavelmente mudança na estrutrua do site.
+        if isinstance(error, (WebDriverException, NoSuchElementException, StaleElementReferenceException, TimeoutException)):
+            return "CRITICAL", "stop"
+
+        if isinstance(error, (ConnectionError, Timeout)):
+            return "WARNING", "retry"
+
+        # Erros de Estrutura (problema de código, provavelmente não recuperável)
+        if isinstance(error, (KeyError, TypeError, AttributeError, NameError)):
+            return "CRITICAL", "stop"
+
+
+        # Erros Funcionais (default)
+        return "WARNING", "retry"
+
